@@ -22,19 +22,16 @@
  * SECTION:element-autovideosink
  * @see_also: autoaudiosink, ximagesink, xvimagesink, sdlvideosink
  *
- * <refsect2>
- * <para>
  * autovideosink is a video sink that automatically detects an appropriate
  * video sink to use.  It does so by scanning the registry for all elements
  * that have <quote>Sink</quote> and <quote>Video</quote> in the class field
  * of their element information, and also have a non-zero autoplugging rank.
- * </para>
+ *
+ * <refsect2>
  * <title>Example launch line</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch -v -m videotestsrc ! autovideosink
- * </programlisting>
- * </para>
+ * ]|
  * </refsect2>
  */
 
@@ -72,7 +69,7 @@ GST_ELEMENT_DETAILS ("Auto video sink",
     "Sink/Video",
     "Wrapper video sink for automatically detected video sink",
     "Ronald Bultje <rbultje@ronald.bitfreak.net>\n"
-    "Jan Schmidt <thaytan@noraisin.net");
+    "Jan Schmidt <thaytan@noraisin.net>");
 
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -160,7 +157,7 @@ gst_auto_video_sink_reset (GstAutoVideoSink * sink)
   sink->kid = gst_element_factory_make ("fakesink", "tempsink");
   gst_bin_add (GST_BIN (sink), sink->kid);
 
-  /* pad */
+  /* pad, setting this target should always work */
   targetpad = gst_element_get_static_pad (sink->kid, "sink");
   gst_ghost_pad_set_target (GST_GHOST_PAD (sink->pad), targetpad);
   gst_object_unref (targetpad);
@@ -355,7 +352,9 @@ gst_auto_video_sink_detect (GstAutoVideoSink * sink)
   /* attach ghost pad */
   GST_DEBUG_OBJECT (sink, "Re-assigning ghostpad");
   targetpad = gst_element_get_static_pad (sink->kid, "sink");
-  gst_ghost_pad_set_target (GST_GHOST_PAD (sink->pad), targetpad);
+  if (!gst_ghost_pad_set_target (GST_GHOST_PAD (sink->pad), targetpad))
+    goto target_failed;
+
   gst_object_unref (targetpad);
   GST_DEBUG_OBJECT (sink, "done changing auto video sink");
 
@@ -368,6 +367,13 @@ no_sink:
         ("Failed to find a supported video sink"));
     return FALSE;
   }
+target_failed:
+  {
+    GST_ELEMENT_ERROR (sink, LIBRARY, INIT, (NULL),
+        ("Failed to set target pad"));
+    gst_object_unref (targetpad);
+    return FALSE;
+  }
 }
 
 static GstStateChangeReturn
