@@ -82,9 +82,6 @@ static gboolean gst_rtp_L16_depay_setcaps (GstBaseRTPDepayload * depayload,
 static GstBuffer *gst_rtp_L16_depay_process (GstBaseRTPDepayload * depayload,
     GstBuffer * buf);
 
-static GstStateChangeReturn gst_rtp_L16_depay_change_state (GstElement *
-    element, GstStateChange transition);
-
 static void
 gst_rtp_L16_depay_base_init (gpointer klass)
 {
@@ -101,16 +98,12 @@ gst_rtp_L16_depay_base_init (gpointer klass)
 static void
 gst_rtp_L16_depay_class_init (GstRtpL16DepayClass * klass)
 {
-  GstElementClass *gstelement_class;
   GstBaseRTPDepayloadClass *gstbasertpdepayload_class;
 
-  gstelement_class = (GstElementClass *) klass;
   gstbasertpdepayload_class = (GstBaseRTPDepayloadClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
 
-  gstelement_class->change_state = gst_rtp_L16_depay_change_state;
-
   gstbasertpdepayload_class->set_caps = gst_rtp_L16_depay_setcaps;
   gstbasertpdepayload_class->process = gst_rtp_L16_depay_process;
 
@@ -269,36 +262,6 @@ empty_packet:
   }
 }
 
-static GstStateChangeReturn
-gst_rtp_L16_depay_change_state (GstElement * element, GstStateChange transition)
-{
-  GstRtpL16Depay *rtpL16depay;
-  GstStateChangeReturn ret;
-
-  rtpL16depay = GST_RTP_L16_DEPAY (element);
-
-  /*
-     switch (transition) {
-     case GST_STATE_CHANGE_NULL_TO_READY:
-     break;
-     default:
-     break;
-     }
-   */
-
-  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
-
-  /*
-     switch (transition) {
-     case GST_STATE_CHANGE_READY_TO_NULL:
-     break;
-     default:
-     break;
-     }
-   */
-  return ret;
-}
-
 gboolean
 gst_rtp_L16_depay_plugin_init (GstPlugin * plugin)
 {
