@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2005> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2005> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -27,6 +27,9 @@
 #include <string.h>
 #include "gstrtpamrdepay.h"
 
+GST_DEBUG_CATEGORY_STATIC (rtpamrdepay_debug);
+#define GST_CAT_DEFAULT (rtpamrdepay_debug)
+
 /* references:
  *
  * RFC 3267 - Real-Time Transport Protocol (RTP) Payload Format and File
@@ -36,10 +39,10 @@
 
 /* elementfactory information */
 static const GstElementDetails gst_rtp_amrdepay_details =
-GST_ELEMENT_DETAILS ("RTP packet depayloader",
+GST_ELEMENT_DETAILS ("RTP AMR depayloader",
     "Codec/Depayloader/Network",
-    "Extracts AMR audio from RTP packets (RFC 3267)",
-    "Wim Taymans <wim@fluendo.com>");
+    "Extracts AMR or AMR-WB audio from RTP packets (RFC 3267)",
+    "Wim Taymans <wim.taymans@gmail.com>");
 
 /* RtpAMRDepay signals and args */
 enum
@@ -58,11 +61,12 @@ enum
  * params see RFC 3267, section 8.1
  */
 static GstStaticPadTemplate gst_rtp_amr_depay_sink_template =
-GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"audio\", "
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
         "clock-rate = (int) 8000, "
         "encoding-name = (string) \"AMR\", "
         "encoding-params = (string) \"1\", "
@@ -70,6 +74,25 @@ GST_STATIC_PAD_TEMPLATE ("sink",
          * GstCaps mapping. */
         "octet-align = (string) \"1\", "
         "crc = (string) { \"0\", \"1\" }, "
+        "robust-sorting = (string) \"0\", " "interleaving = (string) \"0\";"
+        /* following options are not needed for a decoder
+         *
+         "mode-set = (int) [ 0, 7 ], "
+         "mode-change-period = (int) [ 1, MAX ], "
+         "mode-change-neighbor = (boolean) { TRUE, FALSE }, "
+         "maxptime = (int) [ 20, MAX ], "
+         "ptime = (int) [ 20, MAX ]"
+         */
+        "application/x-rtp, "
+        "media = (string) \"audio\", "
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
+        "clock-rate = (int) 16000, "
+        "encoding-name = (string) \"AMR-WB\", "
+        "encoding-params = (string) \"1\", "
+        /* NOTE that all values must be strings in orde to be able to do SDP <->
+         * GstCaps mapping. */
+        "octet-align = (string) \"1\", "
+        "crc = (string) { \"0\", \"1\" }, "
         "robust-sorting = (string) \"0\", " "interleaving = (string) \"0\""
         /* following options are not needed for a decoder
          *
@@ -83,10 +106,11 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     );
 
 static GstStaticPadTemplate gst_rtp_amr_depay_src_template =
-GST_STATIC_PAD_TEMPLATE ("src",
+    GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("audio/AMR, " "channels = (int) 1," "rate = (int) 8000")
+    GST_STATIC_CAPS ("audio/AMR, " "channels = (int) 1," "rate = (int) 8000;"
+        "audio/AMR-WB, " "channels = (int) 1," "rate = (int) 16000")
     );
 
 static gboolean gst_rtp_amr_depay_setcaps (GstBaseRTPDepayload * depayload,
@@ -113,18 +137,17 @@ gst_rtp_amr_depay_base_init (gpointer klass)
 static void
 gst_rtp_amr_depay_class_init (GstRtpAMRDepayClass * klass)
 {
-  GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
   GstBaseRTPDepayloadClass *gstbasertpdepayload_class;
 
-  gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
   gstbasertpdepayload_class = (GstBaseRTPDepayloadClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
 
   gstbasertpdepayload_class->process = gst_rtp_amr_depay_process;
   gstbasertpdepayload_class->set_caps = gst_rtp_amr_depay_setcaps;
+
+  GST_DEBUG_CATEGORY_INIT (rtpamrdepay_debug, "rtpamrdepay", 0,
+      "AMR/AMR-WB RTP Depayloader");
 }
 
 static void
@@ -135,7 +158,6 @@ gst_rtp_amr_depay_init (GstRtpAMRDepay * rtpamrdepay,
 
   depayload = GST_BASE_RTP_DEPAYLOAD (rtpamrdepay);
 
-  depayload->clock_rate = 8000;
   gst_pad_use_fixed_caps (GST_BASE_RTP_DEPAYLOAD_SRCPAD (depayload));
 }
 
@@ -146,13 +168,29 @@ gst_rtp_amr_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   GstCaps *srccaps;
   GstRtpAMRDepay *rtpamrdepay;
   const gchar *params;
-  const gchar *str;
-  gint clock_rate;
+  const gchar *str, *type;
+  gint clock_rate, need_clock_rate;
+  gboolean res;
 
   rtpamrdepay = GST_RTP_AMR_DEPAY (depayload);
 
   structure = gst_caps_get_structure (caps, 0);
 
+  /* figure out the mode first and set the clock rates */
+  if ((str = gst_structure_get_string (structure, "encoding-name"))) {
+    if (strcmp (str, "AMR") == 0) {
+      rtpamrdepay->mode = GST_RTP_AMR_DP_MODE_NB;
+      need_clock_rate = 8000;
+      type = "audio/AMR";
+    } else if (strcmp (str, "AMR-WB") == 0) {
+      rtpamrdepay->mode = GST_RTP_AMR_DP_MODE_WB;
+      need_clock_rate = 16000;
+      type = "audio/AMR-WB";
+    } else
+      goto invalid_mode;
+  } else
+    goto invalid_mode;
+
   if (!(str = gst_structure_get_string (structure, "octet-align")))
     rtpamrdepay->octet_align = FALSE;
   else
@@ -195,13 +233,14 @@ gst_rtp_amr_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   }
 
   if (!gst_structure_get_int (structure, "clock-rate", &clock_rate))
-    clock_rate = 8000;
+    clock_rate = need_clock_rate;
+  depayload->clock_rate = clock_rate;
 
   /* we require 1 channel, 8000 Hz, octet aligned, no CRC,
    * no robust sorting, no interleaving for now */
   if (rtpamrdepay->channels != 1)
     return FALSE;
-  if (clock_rate != 8000)
+  if (clock_rate != need_clock_rate)
     return FALSE;
   if (rtpamrdepay->octet_align != TRUE)
     return FALSE;
@@ -210,46 +249,52 @@ gst_rtp_amr_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   if (rtpamrdepay->interleaving != FALSE)
     return FALSE;
 
-  srccaps = gst_caps_new_simple ("audio/AMR",
+  srccaps = gst_caps_new_simple (type,
       "channels", G_TYPE_INT, rtpamrdepay->channels,
       "rate", G_TYPE_INT, clock_rate, NULL);
-  gst_pad_set_caps (GST_BASE_RTP_DEPAYLOAD_SRCPAD (depayload), srccaps);
+  res = gst_pad_set_caps (GST_BASE_RTP_DEPAYLOAD_SRCPAD (depayload), srccaps);
   gst_caps_unref (srccaps);
 
-  rtpamrdepay->negotiated = TRUE;
+  return res;
 
-  return TRUE;
+  /* ERRORS */
+invalid_mode:
+  {
+    GST_ERROR_OBJECT (rtpamrdepay, "invalid encoding-name");
+    return FALSE;
+  }
 }
 
 /* -1 is invalid */
-static gint frame_size[16] = {
+static gint nb_frame_size[16] = {
   12, 13, 15, 17, 19, 20, 26, 31,
   5, -1, -1, -1, -1, -1, -1, 0
 };
+static gint wb_frame_size[16] = {
+  17, 23, 32, 36, 40, 46, 50, 58,
+  60, -1, -1, -1, -1, -1, -1, 0
+};
 
 static GstBuffer *
 gst_rtp_amr_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
 {
   GstRtpAMRDepay *rtpamrdepay;
   GstBuffer *outbuf = NULL;
+  gint payload_len;
+  gint *frame_size;
 
   rtpamrdepay = GST_RTP_AMR_DEPAY (depayload);
 
-  if (!rtpamrdepay->negotiated)
-    goto not_negotiated;
+  /* setup frame size pointer */
+  if (rtpamrdepay->mode == GST_RTP_AMR_DP_MODE_NB)
+    frame_size = nb_frame_size;
+  else
+    frame_size = wb_frame_size;
 
-  if (!gst_rtp_buffer_validate (buf)) {
-    GST_ELEMENT_WARNING (rtpamrdepay, STREAM, DECODE,
-        (NULL), ("AMR RTP packet did not validate"));
-    goto bad_packet;
-  }
-
-  /* when we get here, 1 channel, 8000 Hz, octet aligned, no CRC, 
+  /* when we get here, 1 channel, 8000/16000 Hz, octet aligned, no CRC, 
    * no robust sorting, no interleaving data is to be depayloaded */
   {
-    gint payload_len;
     guint8 *payload, *p, *dp;
-    guint32 timestamp;
     guint8 CMR;
     gint i, num_packets, num_nonempty_packets;
     gint amr_len;
@@ -258,11 +303,8 @@ gst_rtp_amr_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     payload_len = gst_rtp_buffer_get_payload_len (buf);
 
     /* need at least 2 bytes for the header */
-    if (payload_len < 2) {
-      GST_ELEMENT_WARNING (rtpamrdepay, STREAM, DECODE,
-          (NULL), ("AMR RTP payload too small (%d)", payload_len));
-      goto bad_packet;
-    }
+    if (payload_len < 2)
+      goto too_small;
 
     payload = gst_rtp_buffer_get_payload (buf);
 
@@ -289,11 +331,8 @@ gst_rtp_amr_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
       payload_len -= 1;
       payload += 1;
 
-      if (ILP > ILL) {
-        GST_ELEMENT_WARNING (rtpamrdepay, STREAM, DECODE,
-            (NULL), ("AMR RTP wrong interleaving"));
-        goto bad_packet;
-      }
+      if (ILP > ILL)
+        goto wrong_interleaving;
     }
 
     /* 
@@ -316,11 +355,8 @@ gst_rtp_amr_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
 
       fr_size = frame_size[FT];
       GST_DEBUG_OBJECT (rtpamrdepay, "frame size %d", fr_size);
-      if (fr_size == -1) {
-        GST_ELEMENT_WARNING (rtpamrdepay, STREAM, DECODE,
-            (NULL), ("AMR RTP frame size == -1"));
-        goto bad_packet;
-      }
+      if (fr_size == -1)
+        goto wrong_framesize;
 
       if (fr_size > 0) {
         amr_len += fr_size;
@@ -334,26 +370,15 @@ gst_rtp_amr_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
 
     if (rtpamrdepay->crc) {
       /* data len + CRC len + header bytes should be smaller than payload_len */
-      if (num_packets + num_nonempty_packets + amr_len > payload_len) {
-        GST_ELEMENT_WARNING (rtpamrdepay, STREAM, DECODE,
-            (NULL), ("AMR RTP wrong length 1"));
-        goto bad_packet;
-      }
+      if (num_packets + num_nonempty_packets + amr_len > payload_len)
+        goto wrong_length_1;
     } else {
       /* data len + header bytes should be smaller than payload_len */
-      if (num_packets + amr_len > payload_len) {
-        GST_ELEMENT_WARNING (rtpamrdepay, STREAM, DECODE,
-            (NULL), ("AMR RTP wrong length 2"));
-        goto bad_packet;
-      }
+      if (num_packets + amr_len > payload_len)
+        goto wrong_length_2;
     }
 
-    timestamp = gst_rtp_buffer_get_timestamp (buf);
-
     outbuf = gst_buffer_new_and_alloc (payload_len);
-    GST_BUFFER_TIMESTAMP (outbuf) =
-        gst_util_uint64_scale_int (timestamp, GST_SECOND,
-        depayload->clock_rate);
 
     /* point to destination */
     p = GST_BUFFER_DATA (outbuf);
@@ -379,21 +404,50 @@ gst_rtp_amr_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
         dp += fr_size;
       }
     }
-    gst_buffer_set_caps (outbuf,
-        GST_PAD_CAPS (GST_BASE_RTP_DEPAYLOAD_SRCPAD (depayload)));
+    /* we can set the duration because each packet is 20 milliseconds */
+    GST_BUFFER_DURATION (outbuf) = num_packets * 20 * GST_MSECOND;
 
-    GST_DEBUG ("gst_rtp_amr_depay_chain: pushing buffer of size %d",
+    if (gst_rtp_buffer_get_marker (buf)) {
+      /* marker bit marks a discont buffer after a talkspurt. */
+      GST_DEBUG_OBJECT (depayload, "marker bit was set");
+      GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
+    }
+
+    GST_DEBUG_OBJECT (depayload, "pushing buffer of size %d",
         GST_BUFFER_SIZE (outbuf));
   }
-
   return outbuf;
 
   /* ERRORS */
-not_negotiated:
+too_small:
   {
-    GST_ELEMENT_ERROR (rtpamrdepay, STREAM, NOT_IMPLEMENTED,
-        (NULL), ("not negotiated"));
-    return NULL;
+    GST_ELEMENT_WARNING (rtpamrdepay, STREAM, DECODE,
+        (NULL), ("AMR RTP payload too small (%d)", payload_len));
+    goto bad_packet;
+  }
+wrong_interleaving:
+  {
+    GST_ELEMENT_WARNING (rtpamrdepay, STREAM, DECODE,
+        (NULL), ("AMR RTP wrong interleaving"));
+    goto bad_packet;
+  }
+wrong_framesize:
+  {
+    GST_ELEMENT_WARNING (rtpamrdepay, STREAM, DECODE,
+        (NULL), ("AMR RTP frame size == -1"));
+    goto bad_packet;
+  }
+wrong_length_1:
+  {
+    GST_ELEMENT_WARNING (rtpamrdepay, STREAM, DECODE,
+        (NULL), ("AMR RTP wrong length 1"));
+    goto bad_packet;
+  }
+wrong_length_2:
+  {
+    GST_ELEMENT_WARNING (rtpamrdepay, STREAM, DECODE,
+        (NULL), ("AMR RTP wrong length 2"));
+    goto bad_packet;
   }
 bad_packet:
   {
