@@ -78,12 +78,8 @@ gst_rtp_pcma_pay_base_init (gpointer klass)
 static void
 gst_rtp_pcma_pay_class_init (GstRtpPmcaPayClass * klass)
 {
-  GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
   GstBaseRTPPayloadClass *gstbasertppayload_class;
 
-  gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
   gstbasertppayload_class = (GstBaseRTPPayloadClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
