@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2006> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2006> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -32,10 +32,10 @@ GST_DEBUG_CATEGORY_STATIC (rtptheoradepay_debug);
 
 /* elementfactory information */
 static const GstElementDetails gst_rtp_theora_depay_details =
-GST_ELEMENT_DETAILS ("RTP packet depayloader",
+GST_ELEMENT_DETAILS ("RTP Theora depayloader",
     "Codec/Depayloader/Network",
     "Extracts Theora video from RTP packets (draft-01 of RFC XXXX)",
-    "Wim Taymans <wim@fluendo.com>");
+    "Wim Taymans <wim.taymans@gmail.com>");
 
 static GstStaticPadTemplate gst_rtp_theora_depay_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
@@ -79,9 +79,6 @@ static GstBuffer *gst_rtp_theora_depay_process (GstBaseRTPDepayload * depayload,
 
 static void gst_rtp_theora_depay_finalize (GObject * object);
 
-static GstStateChangeReturn gst_rtp_theora_depay_change_state (GstElement *
-    element, GstStateChange transition);
-
 
 static void
 gst_rtp_theora_depay_base_init (gpointer klass)
@@ -100,17 +97,13 @@ static void
 gst_rtp_theora_depay_class_init (GstRtpTheoraDepayClass * klass)
 {
   GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
   GstBaseRTPDepayloadClass *gstbasertpdepayload_class;
 
   gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
   gstbasertpdepayload_class = (GstBaseRTPDepayloadClass *) klass;
 
   gobject_class->finalize = gst_rtp_theora_depay_finalize;
 
-  gstelement_class->change_state = gst_rtp_theora_depay_change_state;
-
   gstbasertpdepayload_class->process = gst_rtp_theora_depay_process;
   gstbasertpdepayload_class->set_caps = gst_rtp_theora_depay_setcaps;
 
@@ -124,6 +117,7 @@ gst_rtp_theora_depay_init (GstRtpTheoraDepay * rtptheoradepay,
 {
   rtptheoradepay->adapter = gst_adapter_new ();
 }
+
 static void
 gst_rtp_theora_depay_finalize (GObject * object)
 {
@@ -134,54 +128,6 @@ gst_rtp_theora_depay_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static const guint8 a2bin[256] = {
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 62, 64, 64, 64, 63,
-  52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 64, 64, 64, 64, 64, 64,
-  64, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
-  15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 64, 64, 64, 64, 64,
-  64, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
-  41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64
-};
-
-static guint
-decode_base64 (const gchar * in, guint8 * out)
-{
-  guint8 v1, v2;
-  guint len = 0;
-
-  v1 = a2bin[(gint) * in];
-  while (v1 <= 63) {
-    /* read 4 bytes, write 3 bytes, invalid base64 are zeroes */
-    v2 = a2bin[(gint) * ++in];
-    *out++ = (v1 << 2) | ((v2 & 0x3f) >> 4);
-    v1 = (v2 > 63 ? 64 : a2bin[(gint) * ++in]);
-    *out++ = (v2 << 4) | ((v1 & 0x3f) >> 2);
-    v2 = (v1 > 63 ? 64 : a2bin[(gint) * ++in]);
-    *out++ = (v1 << 6) | (v2 & 0x3f);
-    v1 = (v2 > 63 ? 64 : a2bin[(gint) * ++in]);
-    len += 3;
-  }
-  /* move to '\0' */
-  while (*in != '\0')
-    in++;
-
-  /* subtract padding */
-  while (len > 0 && *--in == '=')
-    len--;
-
-  return len;
-}
-
 static gboolean
 gst_rtp_theora_depay_parse_configuration (GstRtpTheoraDepay * rtptheoradepay,
     const gchar * configuration)
@@ -189,17 +135,17 @@ gst_rtp_theora_depay_parse_configuration (GstRtpTheoraDepay * rtptheoradepay,
   GstBuffer *buf;
   guint32 num_headers;
   guint8 *data;
-  guint size;
+  gsize size;
   gint i, j;
 
   /* deserialize base64 to buffer */
   size = strlen (configuration);
-  GST_DEBUG_OBJECT (rtptheoradepay, "base64 config size %u", size);
+  GST_DEBUG_OBJECT (rtptheoradepay, "base64 config size %" G_GSIZE_FORMAT,
+      size);
 
-  data = g_malloc (size);
-  size = decode_base64 (configuration, data);
+  data = g_base64_decode (configuration, &size);
 
-  GST_DEBUG_OBJECT (rtptheoradepay, "config size %u", size);
+  GST_DEBUG_OBJECT (rtptheoradepay, "config size %" G_GSIZE_FORMAT, size);
 
   /* +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    * |                     Number of packed headers                  |
@@ -264,7 +210,8 @@ gst_rtp_theora_depay_parse_configuration (GstRtpTheoraDepay * rtptheoradepay,
     data += 6;
 
     GST_DEBUG_OBJECT (rtptheoradepay,
-        "header %d, ident 0x%08x, length %u, left %u", i, ident, length, size);
+        "header %d, ident 0x%08x, length %u, left %" G_GSIZE_FORMAT, i, ident,
+        length, size);
 
     if (size < length)
       goto too_small;
@@ -331,6 +278,7 @@ gst_rtp_theora_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   GstCaps *srccaps;
   const gchar *delivery_method;
   const gchar *configuration;
+  gboolean res;
 
   rtptheoradepay = GST_RTP_THEORA_DEPAY (depayload);
 
@@ -362,13 +310,13 @@ gst_rtp_theora_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
 
   /* set caps on pad and on header */
   srccaps = gst_caps_new_simple ("video/x-theora", NULL);
-  gst_pad_set_caps (depayload->srcpad, srccaps);
+  res = gst_pad_set_caps (depayload->srcpad, srccaps);
   gst_caps_unref (srccaps);
 
   /* Clock rate is always 90000 according to draft-barbato-avt-rtp-theora-01 */
   depayload->clock_rate = 90000;
 
-  return TRUE;
+  return res;
 
   /* ERRORS */
 unsupported_delivery_method:
@@ -440,13 +388,9 @@ gst_rtp_theora_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
   guint32 timestamp;
   guint32 header, ident;
   guint8 F, TDT, packets;
-  gboolean free_payload;
 
   rtptheoradepay = GST_RTP_THEORA_DEPAY (depayload);
 
-  if (!gst_rtp_buffer_validate (buf))
-    goto bad_packet;
-
   payload_len = gst_rtp_buffer_get_payload_len (buf);
 
   GST_DEBUG_OBJECT (depayload, "got RTP packet of size %d", payload_len);
@@ -456,7 +400,6 @@ gst_rtp_theora_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     goto packet_short;
 
   payload = gst_rtp_buffer_get_payload (buf);
-  free_payload = FALSE;
 
   header = GST_READ_UINT32_BE (payload);
   /*
@@ -614,12 +557,6 @@ no_output:
     return NULL;
   }
   /* ERORRS */
-bad_packet:
-  {
-    GST_ELEMENT_WARNING (rtptheoradepay, STREAM, DECODE,
-        (NULL), ("Packet did not validate"));
-    return NULL;
-  }
 switch_failed:
   {
     GST_ELEMENT_ERROR (rtptheoradepay, STREAM, DECODE,
@@ -645,35 +582,6 @@ length_short:
   }
 }
 
-static GstStateChangeReturn
-gst_rtp_theora_depay_change_state (GstElement * element,
-    GstStateChange transition)
-{
-  GstRtpTheoraDepay *rtptheoradepay;
-  GstStateChangeReturn ret;
-
-  rtptheoradepay = GST_RTP_THEORA_DEPAY (element);
-
-  switch (transition) {
-    case GST_STATE_CHANGE_NULL_TO_READY:
-      break;
-    case GST_STATE_CHANGE_READY_TO_PAUSED:
-      break;
-    default:
-      break;
-  }
-
-  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
-
-  switch (transition) {
-    case GST_STATE_CHANGE_READY_TO_NULL:
-      break;
-    default:
-      break;
-  }
-  return ret;
-}
-
 gboolean
 gst_rtp_theora_depay_plugin_init (GstPlugin * plugin)
 {
