@@ -1142,6 +1142,7 @@ gst_waveparse_ignore_chunk (GstWavParse * wav, GstBuffer * buf, guint32 tag,
 static GstFlowReturn
 gst_wavparse_stream_headers (GstWavParse * wav)
 {
+  GstFlowReturn res;
   GstBuffer *buf;
   gst_riff_strf_auds *header = NULL;
   guint32 tag, size;
@@ -1166,7 +1167,6 @@ gst_wavparse_stream_headers (GstWavParse * wav)
 
       buf = gst_adapter_take_buffer (wav->adapter, size);
     } else {
-      GstFlowReturn res;
       if ((res = gst_riff_read_chunk (GST_ELEMENT_CAST (wav), wav->sinkpad,
                   &wav->offset, &tag, &buf)) != GST_FLOW_OK)
         return res;
@@ -1297,8 +1297,9 @@ gst_wavparse_stream_headers (GstWavParse * wav)
       if (!gst_wavparse_peek_chunk_info (wav, &tag, &size))
         return GST_FLOW_OK;
     } else {
-      if (gst_pad_pull_range (wav->sinkpad, wav->offset, 8,
-              &buf) != GST_FLOW_OK)
+      if ((res =
+              gst_pad_pull_range (wav->sinkpad, wav->offset, 8,
+                  &buf)) != GST_FLOW_OK)
         goto header_read_error;
       tag = GST_READ_UINT32_LE (GST_BUFFER_DATA (buf));
       size = GST_READ_UINT32_LE (GST_BUFFER_DATA (buf) + 4);
@@ -1354,8 +1355,9 @@ gst_wavparse_stream_headers (GstWavParse * wav)
             gst_adapter_flush (wav->adapter, data_size);
           } else {
             gst_buffer_unref (buf);
-            if (gst_pad_pull_range (wav->sinkpad, wav->offset + 8,
-                    data_size, &buf) != GST_FLOW_OK)
+            if ((res =
+                    gst_pad_pull_range (wav->sinkpad, wav->offset + 8,
+                        data_size, &buf)) != GST_FLOW_OK)
               goto header_read_error;
             wav->fact = GST_READ_UINT32_LE (GST_BUFFER_DATA (buf));
             gst_buffer_unref (buf);
@@ -1381,8 +1383,9 @@ gst_wavparse_stream_headers (GstWavParse * wav)
               data_size);
         } else {
           gst_buffer_unref (buf);
-          if (gst_pad_pull_range (wav->sinkpad, wav->offset + 8,
-                  data_size, &buf) != GST_FLOW_OK)
+          if ((res =
+                  gst_pad_pull_range (wav->sinkpad, wav->offset + 8,
+                      data_size, &buf)) != GST_FLOW_OK)
             goto header_read_error;
           acid = (const gst_riff_acid *) GST_BUFFER_DATA (buf);
         }
@@ -1415,8 +1418,9 @@ gst_wavparse_stream_headers (GstWavParse * wav)
           ltag = GST_READ_UINT32_LE (data + 8);
         } else {
           gst_buffer_unref (buf);
-          if (gst_pad_pull_range (wav->sinkpad, wav->offset, 12,
-                  &buf) != GST_FLOW_OK)
+          if ((res =
+                  gst_pad_pull_range (wav->sinkpad, wav->offset, 12,
+                      &buf)) != GST_FLOW_OK)
             goto header_read_error;
           ltag = GST_READ_UINT32_LE (GST_BUFFER_DATA (buf) + 8);
         }
@@ -1425,35 +1429,42 @@ gst_wavparse_stream_headers (GstWavParse * wav)
             const guint data_size = size - 4;
             GstTagList *new;
 
-            GST_INFO_OBJECT (wav, "Have LIST chunk INFO");
+            GST_INFO_OBJECT (wav, "Have LIST chunk INFO size %u", data_size);
             if (wav->streaming) {
               gst_adapter_flush (wav->adapter, 12);
               if (gst_adapter_available (wav->adapter) < data_size) {
                 return GST_FLOW_OK;
               }
               gst_buffer_unref (buf);
-              buf = gst_adapter_take_buffer (wav->adapter, data_size);
+              if (data_size > 0)
+                buf = gst_adapter_take_buffer (wav->adapter, data_size);
             } else {
               wav->offset += 12;
               gst_buffer_unref (buf);
-              if (gst_pad_pull_range (wav->sinkpad, wav->offset, data_size,
-                      &buf) != GST_FLOW_OK)
-                goto header_read_error;
+              if (data_size > 0) {
+                if ((res =
+                        gst_pad_pull_range (wav->sinkpad, wav->offset,
+                            data_size, &buf)) != GST_FLOW_OK)
+                  goto header_read_error;
+              }
             }
-            /* parse tags */
-            gst_riff_parse_info (GST_ELEMENT (wav), buf, &new);
-            if (new) {
-              GstTagList *old = wav->tags;
-              wav->tags = gst_tag_list_merge (old, new, GST_TAG_MERGE_REPLACE);
-              if (old)
-                gst_tag_list_free (old);
-              gst_tag_list_free (new);
-            }
-            if (wav->streaming) {
-              gst_adapter_flush (wav->adapter, data_size);
-            } else {
-              gst_buffer_unref (buf);
-              wav->offset += data_size;
+            if (data_size > 0) {
+              /* parse tags */
+              gst_riff_parse_info (GST_ELEMENT (wav), buf, &new);
+              if (new) {
+                GstTagList *old = wav->tags;
+                wav->tags =
+                    gst_tag_list_merge (old, new, GST_TAG_MERGE_REPLACE);
+                if (old)
+                  gst_tag_list_free (old);
+                gst_tag_list_free (new);
+              }
+              if (wav->streaming) {
+                gst_adapter_flush (wav->adapter, data_size);
+              } else {
+                gst_buffer_unref (buf);
+                wav->offset += data_size;
+              }
             }
             break;
           }
@@ -1578,7 +1589,8 @@ unknown_format:
   }
 header_read_error:
   {
-    GST_ELEMENT_ERROR (wav, STREAM, DEMUX, (NULL), ("Couldn't read in header"));
+    GST_ELEMENT_ERROR (wav, STREAM, DEMUX, (NULL),
+        ("Couldn't read in header %d (%s)", res, gst_flow_get_name (res)));
     g_free (codec_name);
     return GST_FLOW_ERROR;
   }
