@@ -4,7 +4,7 @@
  *               2006 Edgard Lima <edgard.lima@indt.org.br>
  *               2009 Texas Instruments, Inc - http://www.ti.com/
  *
- * gstv4l2bufferpool.c V4L2 buffer pool class
+ * gstv4l2src.h: BT8x8/V4L2 source element
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -30,24 +30,11 @@
 #include <string.h>
 #include <unistd.h>
 
-#include "gst/video/video.h"
-
 #include <gstv4l2bufferpool.h>
 #include "gstv4l2src.h"
-#ifdef HAVE_EXPERIMENTAL
 #include "gstv4l2sink.h"
-#endif
 #include "v4l2_calls.h"
 #include "gst/gst-i18n-plugin.h"
-#include <gst/glib-compat-private.h>
-
-/* videodev2.h is not versioned and we can't easily check for the presence
- * of enum values at compile time, but the V4L2_CAP_VIDEO_OUTPUT_OVERLAY define
- * was added in the same commit as V4L2_FIELD_INTERLACED_{TB,BT} (b2787845) */
-#ifndef V4L2_CAP_VIDEO_OUTPUT_OVERLAY
-#define V4L2_FIELD_INTERLACED_TB 8
-#define V4L2_FIELD_INTERLACED_BT 9
-#endif
 
 
 GST_DEBUG_CATEGORY_EXTERN (v4l2_debug);
@@ -104,9 +91,8 @@ gst_v4l2_buffer_finalize (GstV4l2Buffer * buffer)
   GST_V4L2_BUFFER_POOL_UNLOCK (pool);
 
   if (!resuscitated) {
-    GST_LOG_OBJECT (pool->v4l2elem,
-        "buffer %p (data %p, len %u) not recovered, unmapping",
-        buffer, GST_BUFFER_DATA (buffer), buffer->vbuffer.length);
+    GST_LOG_OBJECT (pool->v4l2elem, "buffer %p not recovered, unmapping",
+        buffer);
     gst_mini_object_unref (GST_MINI_OBJECT (pool));
     v4l2_munmap ((void *) GST_BUFFER_DATA (buffer), buffer->vbuffer.length);
 
@@ -274,7 +260,7 @@ gst_v4l2_buffer_pool_get_type (void)
 
   if (G_UNLIKELY (_gst_v4l2_buffer_pool_type == 0)) {
     static const GTypeInfo v4l2_buffer_pool_info = {
-      sizeof (GstMiniObjectClass),
+      sizeof (GstBufferClass),
       NULL,
       NULL,
       gst_v4l2_buffer_pool_class_init,
@@ -301,10 +287,8 @@ get_v4l2_object (GstElement * v4l2elem)
   GstV4l2Object *v4l2object = NULL;
   if (GST_IS_V4L2SRC (v4l2elem)) {
     v4l2object = (GST_V4L2SRC (v4l2elem))->v4l2object;
-#ifdef HAVE_EXPERIMENTAL
   } else if (GST_IS_V4L2SINK (v4l2elem)) {
     v4l2object = (GST_V4L2SINK (v4l2elem))->v4l2object;
-#endif
   } else {
     GST_ERROR_OBJECT (v4l2elem, "unknown v4l2 element");
   }
@@ -314,19 +298,17 @@ get_v4l2_object (GstElement * v4l2elem)
 
 
 /**
- * gst_v4l2_buffer_pool_new:
- * @v4l2elem:  the v4l2 element (src or sink) that owns this pool
- * @fd:   the video device file descriptor
- * @num_buffers:  the requested number of buffers in the pool
- * @caps:  the caps to set on the buffer
- * @requeuebuf: if %TRUE, and if the pool is still in the running state, a
- *  buffer with no remaining references is immediately passed back to v4l2
- *  (VIDIOC_QBUF), otherwise it is returned to the pool of available buffers
- *  (which can be accessed via gst_v4l2_buffer_pool_get().
+ * Construct a new buffer pool
  *
- * Construct a new buffer pool.
- *
- * Returns: the new pool, use gst_v4l2_buffer_pool_destroy() to free resources
+ * @v4l2elem  the v4l2 element (src or sink) that owns this pool
+ * @fd   the video device file descriptor
+ * @num_buffers  the requested number of buffers in the pool
+ * @caps  the caps to set on the buffer
+ * @requeuebuf  if <code>TRUE</code>, and if the pool is still in the
+ *   <code>running</code> state, a buffer with no remaining references
+ *   is immediately passed back to v4l2 (<code>VIDIOC_QBUF</code>),
+ *   otherwise it is returned to the pool of available buffers
+ *   (which can be accessed via <code>gst_v4l2_buffer_pool_get()</code>.
  */
 GstV4l2BufferPool *
 gst_v4l2_buffer_pool_new (GstElement * v4l2elem, gint fd, gint num_buffers,
@@ -427,12 +409,7 @@ buffer_new_failed:
   }
 }
 
-/**
- * gst_v4l2_buffer_pool_destroy:
- * @pool: the pool
- *
- * Free all resources in the pool and the pool itself.
- */
+
 void
 gst_v4l2_buffer_pool_destroy (GstV4l2BufferPool * pool)
 {
@@ -466,30 +443,17 @@ gst_v4l2_buffer_pool_destroy (GstV4l2BufferPool * pool)
 }
 
 /**
- * gst_v4l2_buffer_pool_get:
- * @pool:   the "this" object
- * @blocking:  should this call suspend until there is a buffer available
- *    in the buffer pool?
- *
  * Get an available buffer in the pool
+ *
+ * @pool   the "this" object
  */
 GstV4l2Buffer *
-gst_v4l2_buffer_pool_get (GstV4l2BufferPool * pool, gboolean blocking)
+gst_v4l2_buffer_pool_get (GstV4l2BufferPool * pool)
 {
-  GstV4l2Buffer *buf;
+  GstV4l2Buffer *buf = g_async_queue_try_pop (pool->avail_buffers);
 
-  if (blocking) {
-    buf = g_async_queue_pop (pool->avail_buffers);
-  } else {
-    buf = g_async_queue_try_pop (pool->avail_buffers);
-  }
-
-  if (buf) {
-    GST_V4L2_BUFFER_POOL_LOCK (pool);
+  if (buf)
     GST_BUFFER_SIZE (buf) = buf->vbuffer.length;
-    GST_BUFFER_FLAG_UNSET (buf, 0xffffffff);
-    GST_V4L2_BUFFER_POOL_UNLOCK (pool);
-  }
 
   pool->running = TRUE;
 
@@ -498,13 +462,10 @@ gst_v4l2_buffer_pool_get (GstV4l2BufferPool * pool, gboolean blocking)
 
 
 /**
- * gst_v4l2_buffer_pool_qbuf:
- * @pool: the pool
- * @buf: the buffer to queue
- *
  * Queue a buffer to the driver
  *
- * Returns: %TRUE for success
+ * @pool   the "this" object
+ * @buf    the buffer to queue
  */
 gboolean
 gst_v4l2_buffer_pool_qbuf (GstV4l2BufferPool * pool, GstV4l2Buffer * buf)
@@ -522,14 +483,11 @@ gst_v4l2_buffer_pool_qbuf (GstV4l2BufferPool * pool, GstV4l2Buffer * buf)
 }
 
 /**
- * gst_v4l2_buffer_pool_dqbuf:
- * @pool: the pool
- *
  * Dequeue a buffer from the driver.  Some generic error handling is done in
  * this function, but any error handling specific to v4l2src (capture) or
  * v4l2sink (output) can be done outside this function by checking 'errno'
  *
- * Returns: a buffer
+ * @pool   the "this" object
  */
 GstV4l2Buffer *
 gst_v4l2_buffer_pool_dqbuf (GstV4l2BufferPool * pool)
@@ -570,17 +528,11 @@ gst_v4l2_buffer_pool_dqbuf (GstV4l2BufferPool * pool)
     GST_DEBUG_OBJECT (pool->v4l2elem, "num_live_buffers++: %d",
         pool->num_live_buffers);
 
-    /* set top/bottom field first if v4l2_buffer has the information */
-    if (buffer.field == V4L2_FIELD_INTERLACED_TB)
-      GST_BUFFER_FLAG_SET (pool_buffer, GST_VIDEO_BUFFER_TFF);
-    if (buffer.field == V4L2_FIELD_INTERLACED_BT)
-      GST_BUFFER_FLAG_UNSET (pool_buffer, GST_VIDEO_BUFFER_TFF);
+    GST_V4L2_BUFFER_POOL_UNLOCK (pool);
 
     /* this can change at every frame, esp. with jpeg */
     GST_BUFFER_SIZE (pool_buffer) = buffer.bytesused;
 
-    GST_V4L2_BUFFER_POOL_UNLOCK (pool);
-
     return pool_buffer;
   }
 
@@ -638,12 +590,8 @@ gst_v4l2_buffer_pool_dqbuf (GstV4l2BufferPool * pool)
 
 /**
  * gst_v4l2_buffer_pool_available_buffers:
- * @pool: the pool
- *
- * Check the number of buffers available to the driver, ie. buffers that
+ * Returns the number of buffers available to the driver, ie. buffers that
  * have been QBUF'd but not yet DQBUF'd.
- *
- * Returns: the number of buffers available.
  */
 gint
 gst_v4l2_buffer_pool_available_buffers (GstV4l2BufferPool * pool)
