@@ -25,18 +25,20 @@
 #define __V4L2_CALLS_H__
 
 #include "gstv4l2object.h"
-#include "gst/gst-i18n-plugin.h"
 
 #ifdef HAVE_LIBV4L2
-#include <libv4l2.h>
+#  include <libv4l2.h>
 #else
-#define v4l2_fd_open(fd, flags) (fd)
-#define v4l2_close close
-#define v4l2_dup dup
-#define v4l2_ioctl ioctl
-#define v4l2_read read
-#define v4l2_mmap mmap
-#define v4l2_munmap munmap
+#  include <sys/ioctl.h>
+#  include <linux/videodev.h>
+#  include <linux/videodev2.h>
+#  define v4l2_fd_open(fd, flags) (fd)
+#  define v4l2_close    close
+#  define v4l2_dup      dup
+#  define v4l2_ioctl    ioctl
+#  define v4l2_read     read
+#  define v4l2_mmap     mmap
+#  define v4l2_munmap   munmap
 #endif
 
 /* simple check whether the device is open */
@@ -137,4 +139,7 @@ gboolean	gst_v4l2_set_attribute		(GstV4l2Object *v4l2object,
 
 gboolean        gst_v4l2_get_capabilities       (GstV4l2Object * v4l2object);
 
+
+#define LOG_CAPS(obj, caps) GST_DEBUG_OBJECT (obj, "%s: %" GST_PTR_FORMAT, #caps, caps)
+
 #endif /* __V4L2_CALLS_H__ */
