@@ -67,7 +67,8 @@ setup_src_pad (GstElement * element,
   ASSERT_OBJECT_REFCOUNT (srcpad, "srcpad", 1);
   gst_pad_set_active (srcpad, TRUE);
 
-  sinkpad = gst_element_get_pad (element, "audio_%d");
+  if (!(sinkpad = gst_element_get_static_pad (element, "audio_%d")))
+    sinkpad = gst_element_get_request_pad (element, "audio_%d");
   fail_if (sinkpad == NULL, "Could not get sink pad from %s",
       GST_ELEMENT_NAME (element));
   /* references are owned by: 1) us, 2) matroskamux, 3) collect pads */
@@ -90,14 +91,16 @@ teardown_src_pad (GstElement * element)
   GstPad *srcpad, *sinkpad;
 
   /* clean up floating src pad */
-  sinkpad = gst_element_get_pad (element, "audio_0");
-  ASSERT_OBJECT_REFCOUNT (sinkpad, "sinkpad", 2);
+  if (!(sinkpad = gst_element_get_static_pad (element, "audio_0")))
+    sinkpad = gst_element_get_request_pad (element, "audio_0");
+  /* references are owned by: 1) us, 2) matroskamux, 3) collect pads */
+  ASSERT_OBJECT_REFCOUNT (sinkpad, "sinkpad", 3);
   srcpad = gst_pad_get_peer (sinkpad);
 
   gst_pad_unlink (srcpad, sinkpad);
 
-  /* pad refs held by both creator and this function (through _get) */
-  ASSERT_OBJECT_REFCOUNT (sinkpad, "sinkpad", 2);
+  /* references are owned by: 1) us, 2) matroskamux, 3) collect pads */
+  ASSERT_OBJECT_REFCOUNT (sinkpad, "sinkpad", 3);
   gst_object_unref (sinkpad);
   /* one more ref is held by element itself */
 
@@ -120,7 +123,7 @@ setup_sink_pad (GstElement * element, GstStaticPadTemplate * template,
   fail_if (sinkpad == NULL, "Could not create a sinkpad");
   gst_pad_set_active (sinkpad, TRUE);
 
-  srcpad = gst_element_get_pad (element, "src");
+  srcpad = gst_element_get_static_pad (element, "src");
   fail_if (srcpad == NULL, "Could not get source pad from %s",
       GST_ELEMENT_NAME (element));
   if (caps)
@@ -141,7 +144,7 @@ teardown_sink_pad (GstElement * element)
   GstPad *srcpad, *sinkpad;
 
   /* clean up floating sink pad */
-  srcpad = gst_element_get_pad (element, "src");
+  srcpad = gst_element_get_static_pad (element, "src");
   sinkpad = gst_pad_get_peer (srcpad);
   gst_pad_unlink (srcpad, sinkpad);
 
